@@ -841,10 +841,11 @@ class CORE_EXPORT QgsLayoutItem : public QgsLayoutObject, public QGraphicsRectIt
     /**
      * Returns the background color for this item. This is only used if hasBackground()
      * returns TRUE.
+     * \param useDataDefined If true, then returns the data defined override for the background color
      * \see setBackgroundColor()
      * \see hasBackground()
      */
-    QColor backgroundColor() const { return mBackgroundColor; }
+    QColor backgroundColor( bool useDataDefined = true ) const;
 
     /**
      * Sets the background \a color for this item.
@@ -1012,6 +1013,13 @@ class CORE_EXPORT QgsLayoutItem : public QgsLayoutObject, public QGraphicsRectIt
      */
     virtual QgsGeometry clipPath() const;
 
+    /**
+     * Returns TRUE if the item is currently refreshing content in the background.
+     *
+     * \since QGIS 3.32
+     */
+    virtual bool isRefreshing() const;
+
   public slots:
 
     /**
@@ -1136,6 +1144,13 @@ class CORE_EXPORT QgsLayoutItem : public QgsLayoutObject, public QGraphicsRectIt
      */
     virtual void drawBackground( QgsRenderContext &context );
 
+    /**
+     * Draws a "refreshing" overlay icon on the item.
+     *
+     * \since QGIS 3.32
+     */
+    void drawRefreshingOverlay( QPainter *painter, const QStyleOptionGraphicsItem *itemStyle );
+
     /**
      * Sets a fixed \a size for the layout item, which prevents it from being freely
      * resized. Set an empty size if item can be freely resized.
