@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 """
 ***************************************************************************
     hillshade.py
@@ -21,11 +19,6 @@ __author__ = 'Alexander Bruy'
 __date__ = 'October 2013'
 __copyright__ = '(C) 2013, Alexander Bruy'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = '58734527abdbaab95719a48a4a64ea05abd70b44'
-
-
 import os
 
 from qgis.core import (QgsRasterFileWriter,
@@ -44,7 +37,6 @@ pluginPath = os.path.split(os.path.split(os.path.dirname(__file__))[0])[0]
 
 
 class hillshade(GdalAlgorithm):
-
     INPUT = 'INPUT'
     BAND = 'BAND'
     COMPUTE_EDGES = 'COMPUTE_EDGES'
@@ -56,6 +48,7 @@ class hillshade(GdalAlgorithm):
     COMBINED = 'COMBINED'
     MULTIDIRECTIONAL = 'MULTIDIRECTIONAL'
     OPTIONS = 'OPTIONS'
+    EXTRA = 'EXTRA'
     OUTPUT = 'OUTPUT'
 
     def __init__(self):
@@ -111,6 +104,13 @@ class hillshade(GdalAlgorithm):
                 'class': 'processing.algs.gdal.ui.RasterOptionsWidget.RasterOptionsWidgetWrapper'}})
         self.addParameter(options_param)
 
+        extra_param = QgsProcessingParameterString(self.EXTRA,
+                                                   self.tr('Additional command-line parameters'),
+                                                   defaultValue=None,
+                                                   optional=True)
+        extra_param.setFlags(extra_param.flags() | QgsProcessingParameterDefinition.FlagAdvanced)
+        self.addParameter(extra_param)
+
         self.addParameter(QgsProcessingParameterRasterDestination(self.OUTPUT, self.tr('Hillshade')))
 
     def name(self):
@@ -150,7 +150,7 @@ class hillshade(GdalAlgorithm):
         arguments.append('-s')
         arguments.append(str(self.parameterAsDouble(parameters, self.SCALE, context)))
 
-        multidirectional = self.parameterAsBool(parameters, self.MULTIDIRECTIONAL, context)
+        multidirectional = self.parameterAsBoolean(parameters, self.MULTIDIRECTIONAL, context)
         # azimuth and multidirectional are mutually exclusive
         if not multidirectional:
             arguments.append('-az')
@@ -159,21 +159,27 @@ class hillshade(GdalAlgorithm):
         arguments.append('-alt')
         arguments.append(str(self.parameterAsDouble(parameters, self.ALTITUDE, context)))
 
-        if self.parameterAsBool(parameters, self.COMPUTE_EDGES, context):
+        if self.parameterAsBoolean(parameters, self.COMPUTE_EDGES, context):
             arguments.append('-compute_edges')
 
-        if self.parameterAsBool(parameters, self.ZEVENBERGEN, context):
+        if self.parameterAsBoolean(parameters, self.ZEVENBERGEN, context):
             arguments.append('-alg')
             arguments.append('ZevenbergenThorne')
 
-        if self.parameterAsBool(parameters, self.COMBINED, context):
+        combined = self.parameterAsBoolean(parameters, self.COMBINED, context)
+        if combined and not multidirectional:
             arguments.append('-combined')
-
-        if multidirectional:
+        elif multidirectional and not combined:
             arguments.append('-multidirectional')
+        elif multidirectional and combined:
+            raise QgsProcessingException(self.tr('Options -multirectional and -combined are mutually exclusive.'))
 
         options = self.parameterAsString(parameters, self.OPTIONS, context)
         if options:
             arguments.extend(GdalUtils.parseCreationOptions(options))
 
+        if self.EXTRA in parameters and parameters[self.EXTRA] not in (None, ''):
+            extra = self.parameterAsString(parameters, self.EXTRA, context)
+            arguments.append(extra)
+
         return [self.commandName(), GdalUtils.escapeAndJoin(arguments)]
