@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 """
 ***************************************************************************
     Buffer.py
@@ -21,11 +19,8 @@ __author__ = 'Victor Olaya'
 __date__ = 'August 2012'
 __copyright__ = '(C) 2012, Victor Olaya'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = '26842169e9367434a27a70ab0df0bd4ba6ad1d7a'
-
-from qgis.core import (QgsFeature,
+from qgis.core import (Qgis,
+                       QgsFeature,
                        QgsGeometry,
                        QgsFeatureRequest,
                        QgsFeatureSink)
@@ -33,7 +28,6 @@ from qgis.core import (QgsFeature,
 
 def buffering(feedback, context, sink, distance, field, useField, source, dissolve, segments, endCapStyle=1,
               joinStyle=1, miterLimit=2):
-
     if useField:
         field = source.fields().lookupField(field)
 
@@ -62,7 +56,7 @@ def buffering(feedback, context, sink, distance, field, useField, source, dissol
 
             inGeom = inFeat.geometry()
 
-            buffered_geometries.append(inGeom.buffer(float(value), segments, endCapStyle, joinStyle, miterLimit))
+            buffered_geometries.append(inGeom.buffer(float(value), segments, Qgis.EndCapStyle(endCapStyle), Qgis.JoinStyle(joinStyle), miterLimit))
 
             current += 1
             feedback.setProgress(int(current * total))
@@ -86,7 +80,7 @@ def buffering(feedback, context, sink, distance, field, useField, source, dissol
                 value = distance
             inGeom = inFeat.geometry()
             outFeat = QgsFeature()
-            outGeom = inGeom.buffer(float(value), segments, endCapStyle, joinStyle, miterLimit)
+            outGeom = inGeom.buffer(float(value), segments, Qgis.EndCapStyle(endCapStyle), Qgis.JoinStyle(joinStyle), miterLimit)
             outFeat.setGeometry(outGeom)
             outFeat.setAttributes(attrs)
             sink.addFeature(outFeat, QgsFeatureSink.FastInsert)
