@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 """
 ***************************************************************************
     PointSelectionPanel.py
@@ -21,10 +19,6 @@ __author__ = 'Alexander Bruy'
 __date__ = 'February 2016'
 __copyright__ = '(C) 2016, Alexander Bruy'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = '0c5774c0685d4b6f3e92a086234455708d1cd427'
-
 import os
 import warnings
 
@@ -48,7 +42,7 @@ with warnings.catch_warnings():
 class PointSelectionPanel(BASE, WIDGET):
 
     def __init__(self, dialog, default=None):
-        super(PointSelectionPanel, self).__init__(None)
+        super().__init__(None)
         self.setupUi(self)
 
         self.btnSelect.clicked.connect(self.selectOnCanvas)
@@ -83,7 +77,7 @@ class PointSelectionPanel(BASE, WIDGET):
         self.dialog.showMinimized()
 
     def updatePoint(self, point, button):
-        s = '{},{}'.format(point.x(), point.y())
+        s = f'{point.x()},{point.y()}'
         self.crs = QgsProject.instance().crs()
         if self.crs.isValid():
             s += ' [' + self.crs.authid() + ']'
